package me.paradoxpixel.themepark.attraction;

import me.paradoxpixel.themepark.ThemeParkPlugin;
import me.paradoxpixel.themepark.action.TPUtils;
import me.paradoxpixel.themepark.api.API;
import me.paradoxpixel.themepark.api.attraction.Attraction;
import me.paradoxpixel.themepark.api.attraction.Region;
import me.paradoxpixel.themepark.api.attraction.component.Status;
import me.paradoxpixel.themepark.api.attraction.component.Type;
import me.paradoxpixel.themepark.attraction.status.StatusManager;
import me.paradoxpixel.themepark.config.YamlConfig;
import me.paradoxpixel.themepark.gui.GUI;
import me.paradoxpixel.themepark.gui.GUIItem;
import me.paradoxpixel.themepark.utils.ItemBuilder;
import me.paradoxpixel.themepark.utils.LocationUtils;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.entity.Player;
import java.util.HashMap;
import java.util.List;

public class AttractionMenu {

    private static YamlConfig config = ThemeParkPlugin.getInstance().getAttraction();
    private static YamlConfig settings = ThemeParkPlugin.getInstance().getSettings();
    private static GUI gui;
    private static boolean loading = false;

    private static HashMap<String, Integer> index;

    public static void load() {
        loading = true;
        gui = new GUI(settings.getConfig().getString("menu.title"), 9);
        index = new HashMap<>();
        loadData();
        loading = false;
        loadItems();
    }

    public static void reload() {
        loadItems();
    }

    public static void save() {
        for(Region region : API.getRegions().values()) {
            config.getConfig().set("region." + region.getId() + ".name", region.getName());
            config.getConfig().set("region." + region.getId() + ".lore", region.getLore());
        }

        for(Attraction attraction : API.getAttractions().values()) {
            config.getConfig().set("attraction." + attraction.getId() + ".name", attraction.getName());
            config.getConfig().set("attraction." + attraction.getId() + ".region_id", attraction.getRegion_id());
            config.getConfig().set("attraction." + attraction.getId() + ".type", attraction.getType().toString());
            config.getConfig().set("attraction." + attraction.getId() + ".status", attraction.getStatus().toString());
            config.getConfig().set("attraction." + attraction.getId() + ".location", LocationUtils.toString(attraction.getLocation()));
        }

        config.save();
    }

    private static void loadData() {
        if(!config.getConfig().contains("region"))
            return;

        if(!config.getConfig().isConfigurationSection("region"))
            return;

        ConfigurationSection section = config.getConfig().getConfigurationSection("region");
        if(section.getKeys(false).isEmpty())
            return;

        HashMap<String, Region> regions = API.getRegions();
        for(String id : section.getKeys(false)) {
            if(API.getRegions().size() >= 6) {
                config.getConfig().set("regions." + id, null);
                config.save();
                continue;
            }

            String name = config.getConfig().getString("region." + id + ".name");
            List<String> lore = config.getConfig().getStringList("region." + id + ".lore");

            regions.remove(id);
            if(API.isRegion(id)) {
                API.getRegion(id).setName(name);
                API.getRegion(id).setLore(lore);
                continue;
            }

            Region region = new Region(id, name, lore);
            API.addRegion(region);
        }

        for(String string : regions.keySet())
            API.removeRegion(string);

        if(!config.getConfig().contains("attraction"))
            return;

        if(!config.getConfig().isConfigurationSection("attraction"))
            return;

        section = config.getConfig().getConfigurationSection("attraction");
        if(section.getKeys(false).isEmpty())
            return;

        HashMap<String, Attraction> attractions = API.getAttractions();
        for(String id : section.getKeys(false)) {
            String region_id = config.getConfig().getString("attraction." + id + ".region_id");
            if(!API.isRegion(region_id))
                continue;

            if(API.isRegionFull(region_id))
                continue;

            String name = config.getConfig().getString("attraction." + id + ".name");
            Location location = LocationUtils.toLocation(config.getConfig().getString("attraction." + id + ".location"));
            Type type = Type.getType(config.getConfig().getString("attraction." + id + ".type"));
            Status status = Status.getStatus(config.getConfig().getString("attraction." + id + ".status"));

            attractions.remove(id);
            if(API.isAttraction(id)) {
                Attraction attraction = API.getAttraction(id);
                attraction.setName(name);
                attraction.setRegion_id(region_id);
                attraction.setLocation(location);
                attraction.setType(type);
                attraction.setStatus(status, null);
                if(!type.containsStatus(status)) {
                    config.getConfig().set("attraction." + id + ".status", type.getDefault().toString());
                    config.save();
                }
                continue;
            }

            Attraction attraction = new Attraction(id, name, region_id, location, type, status);
            API.addAttraction(attraction);
        }

        for(String string : attractions.keySet())
            API.removeAttraction(string);
    }

    private static void loadItems() {
        if(loading)
            return;

        if(API.getRegions().values().size() == 0)
            return;

        gui.clear();
        gui.setSize(API.getRegions().size() > 0 ? (API.getRegions().size() * 9) : 9);
        for(Region region : API.getRegions().values()) {
            if(!index.containsKey(region.getId()))
                index.put(region.getId(), index.size() * 9);

            int i = index.get(region.getId());
            ItemBuilder builder = new ItemBuilder(Material.NAME_TAG);
            builder.setName(region.getName());
            builder.setLore(region.getLore());

            gui.addItem(i, new GUIItem(builder.getItem(), null, true));
        }

        for(Attraction attraction : API.getAttractions().values()) {
            if(!API.isRegion(attraction.getRegion_id())) {
                API.removeAttraction(attraction.getId());
                continue;
            }

            if(!index.containsKey(attraction.getRegion_id()))
                continue;

            int i = index.get(attraction.getRegion_id());
            boolean b = false;
            for(int a = 1; a < 9; a++) {
                if(!gui.hasItem(i + a)) {
                    i += a;
                    b = true;
                    break;
                }
            }

            if(!b) {
                API.removeAttraction(attraction.getId());
                continue;
            }

            ItemBuilder builder = new ItemBuilder(StatusManager.getMaterial(attraction.getStatus()), 1, StatusManager.getData(attraction.getStatus()));
            builder.setName(attraction.getName());
            builder.setLore(StatusManager.getName(attraction.getStatus()));

            gui.addItem(i, new GUIItem(builder.getItem(), new TPUtils(attraction.getId()), true));
        }
    }

    public static void openMenu(Player player) {
        player.openInventory(gui.getInventory());
    }

}