package me.paradoxpixel.themepark.attraction.sign;

import me.paradoxpixel.themepark.ThemeParkPlugin;
import me.paradoxpixel.themepark.api.attraction.Attraction;
import me.paradoxpixel.themepark.config.YamlConfig;
import me.paradoxpixel.themepark.utils.LocationUtils;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.block.Sign;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class SignManager {

    private static YamlConfig config = ThemeParkPlugin.getInstance().getSigns();
    private static HashMap<Attraction, ArrayList<StatusSign>> signs = new HashMap<>();

    public static void loadSigns(Attraction attraction) {
        if(attraction == null)
            return;

        if(!config.getConfig().contains("signs." + attraction.getId()))
            return;

        if(!config.getConfig().isList("signs." + attraction.getId())) {
            config.getConfig().set("signs." + attraction.getId(), null);
            config.save();
            return;
        }

        ArrayList<StatusSign> array = new ArrayList<>();
        List<String> locations = config.getConfig().getStringList("signs." + attraction.getId());
        for(String string : locations) {
            Location location = LocationUtils.toLocation(string);
            if(location == null)
                continue;

            if(!(location.getBlock().getState() instanceof Sign))
                return;

            array.add(new StatusSign(attraction, location));
        }

        signs.put(attraction, array);
    }

    public static void saveSigns() {
        for(Map.Entry<Attraction, ArrayList<StatusSign>> entry : signs.entrySet()) {
            List<String> array = new ArrayList<>();
            for(StatusSign sign : entry.getValue())
                array.add(LocationUtils.toString(sign.getLocation()));

            config.getConfig().set("signs." + entry.getKey().getId(), new ArrayList<>());
            config.getConfig().set("signs." + entry.getKey().getId(), array);
            config.save();
        }
    }

    public static void addSign(StatusSign sign) {
        if(sign == null || sign.getLocation() == null)
            return;

        if(!(sign.getLocation().getBlock().getState() instanceof Sign))
            return;

        if(signs.containsKey(sign.getAttraction())) {
            signs.get(sign.getAttraction()).add(sign);
            return;
        }

        ArrayList<StatusSign> array = new ArrayList<>();
        array.add(sign);
        signs.put(sign.getAttraction(), array);
    }

    public static boolean hasSigns(Attraction attraction) {
        if(attraction == null)
            return false;

        return signs.containsKey(attraction) && (signs.get(attraction).size() > 0);
    }

    public static void update(Attraction attraction) {
        if(attraction == null)
            return;

        if(!hasSigns(attraction))
            return;

        for(StatusSign sign : signs.get(attraction))
            sign.update();
    }

    public static void remove(Attraction attraction) {
        if(attraction == null || !signs.containsKey(attraction))
            return;

        for(StatusSign sign : signs.get(attraction))
            sign.getLocation().getBlock().setType(Material.AIR);

        signs.remove(attraction);
        config.getConfig().set("signs." + attraction.getId(), null);
        config.save();
    }

    public static void removeSign(StatusSign sign) {
        if(sign == null || sign.getLocation() == null || sign.getAttraction() == null)
            return;

        if(!hasSigns(sign.getAttraction()))
            return;

        ArrayList<StatusSign> array = signs.get(sign.getAttraction());
        for(StatusSign s : new ArrayList<>(array)) {
            Location loc1 = sign.getLocation();
            Location loc2 = s.getLocation();

            if(loc1 == null || loc2 == null)
                return;

            if(LocationUtils.toString(loc1) == null || LocationUtils.toString(loc2) == null)
                return;

            if(!LocationUtils.toString(loc1).equals(LocationUtils.toString(loc2)))
                return;

            array.remove(s);
        }

        if(array.isEmpty()) {
            signs.remove(sign.getAttraction());
            config.getConfig().set("signs." + sign.getAttraction().getId(), null);
            config.save();
            return;
        }

        signs.put(sign.getAttraction(), array);
    }

}
