package me.paradoxpixel.themepark.database;

import me.paradoxpixel.themepark.api.API;
import me.paradoxpixel.themepark.api.attraction.Attraction;
import me.paradoxpixel.themepark.api.attraction.Region;
import me.paradoxpixel.themepark.api.attraction.component.Status;
import me.paradoxpixel.themepark.attraction.status.StatusManager;
import me.paradoxpixel.themepark.config.YamlConfig;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.UUID;

public class DatabaseHandler {

    private Database database;

    public DatabaseHandler(YamlConfig config) {
        database = new Database(config);
        load();
    }

    public void load() {
        database.execute("CREATE TABLE IF NOT EXISTS attraction (id VARCHAR(256) NOT NULL, name VARCHAR(256) NOT NULL, region_id VARCHAR(256) NOT NULL, type VARCHAR(256) NOT NULL, status VARCHAR(256) NOT NULL, PRIMARY KEY(id))", null);
        database.execute("CREATE TABLE IF NOT EXISTS region (id VARCHAR(256) NOT NULL, name VARCHAR(256) NOT NULL, PRIMARY KEY(id))", null);
        database.execute("CREATE TABLE IF NOT EXISTS status (statusId VARCHAR(256) NOT NULL, statusName VARCHAR(256) NOT NULL)", null);

        database.execute("TRUNCATE TABLE attraction", null);
        database.execute("TRUNCATE TABLE region", null);
        database.execute("TRUNCATE TABLE status", null);

        for(Attraction attraction : API.getAttractions().values())
            addAttraction(attraction);

        for(Region region : API.getRegions().values())
            addRegion(region);

        for(Status status : Status.values()) {
            HashMap<Integer, Object> objects = new HashMap<>();
            objects.put(1, status.toString());
            objects.put(2, StatusManager.getName(status));
            database.execute("INSERT INTO status (statusId,statusName) VALUES(?,?)", objects);
        }
    }

    public void addAttraction(Attraction attraction) {
        HashMap<Integer, Object> objects = new HashMap<>();
        objects.put(1, attraction.getId());
        objects.put(2, attraction.getName());
        objects.put(3, attraction.getRegion_id());
        objects.put(4, attraction.getType().toString());
        objects.put(5, attraction.getStatus().toString());

        database.execute("INSERT IGNORE INTO attraction(id, name, region_id, type, status) VALUES (?,?,?,?,?)", objects);
    }

    public void updateAttraction(Attraction attraction) {
        HashMap<Integer, Object> objects = new HashMap<>();
        objects.put(1, attraction.getStatus().toString());
        objects.put(2, attraction.getId());

        database.execute("UPDATE attraction SET status=? WHERE id=?", objects);
    }

    public void removeAttraction(Attraction attraction) {
        HashMap<Integer, Object> objects = new HashMap<>();
        objects.put(1, attraction.getId());

        database.execute("DELETE FROM attraction WHERE id=?", objects);
        database.execute("DELETE FROM ridecount WHERE attractionId=?", objects);
    }

    public void addRegion(Region region) {
        HashMap<Integer, Object> objects = new HashMap<>();
        objects.put(1, region.getId());
        objects.put(2, region.getName());

        database.execute("INSERT IGNORE INTO region(id, name) VALUES (?,?)", objects);
    }

    public void updateRegion(Region region) {
        HashMap<Integer, Object> objects = new HashMap<>();
        objects.put(1, region.getName());
        objects.put(2, region.getId());

        database.execute("UPDATE region SET name=? WHERE id=?", objects);
    }

    public void removeRegion(Region region) {
        HashMap<Integer, Object> objects = new HashMap<>();
        objects.put(1, region.getId());

        database.execute("DELETE FROM region WHERE id=?", objects);
    }

    public void disconnect() {
        database.disconnect();
    }

}
