package me.paradoxpixel.themepark.gui;

import me.paradoxpixel.themepark.utils.Utils;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.ClickType;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

public class GUI extends GUIHolder {

    private String title;
    private int size;
    private Inventory inventory;
    private HashMap<Integer, GUIItem> items = new HashMap<>();

    public GUI(String title, int size) {
        if(size > 54)
            size = 54;

        this.title = title;
        this.size = Utils.toSize(size, 9);
        inventory = Bukkit.createInventory(this, this.size, Utils.color(title));
    }

    private void update() {
        ArrayList<Integer> toRemove = new ArrayList<>();
        for(Map.Entry<Integer, GUIItem> entry : items.entrySet()) {
            if(entry.getKey() >= getSize()) {
                toRemove.add(entry.getKey());
                continue;
            }

            inventory.setItem(entry.getKey(), entry.getValue().getItem());
        }

        for(int i : toRemove)
            items.remove(i);
    }

    public void setSize(int size) {
        if(size > 54)
            size = 54;

        this.size = Utils.toSize(size, 9);
        inventory = Bukkit.createInventory(this, this.size, Utils.color(title));
        update();
    }

    public int getSize() {
        return size;
    }

    public void clear() {
        inventory = Bukkit.createInventory(this, Utils.toSize(size, 9), Utils.color(title));
        items.clear();
    }

    public void addItems(HashMap<Integer, GUIItem> items) {
        for(Map.Entry<Integer, GUIItem> entry : items.entrySet())
            addItem(entry.getKey(), entry.getValue());
    }

    public void addItem(int slot, GUIItem item) {
        if(slot >= getSize() || slot < 0)
            return;

        items.put(slot, item);
        inventory.setItem(slot, item.getItem());
    }

    public void removeItem(int slot) {
        items.remove(slot);
    }

    public void switchSlot(int from, int to) {
        if(!items.containsKey(from))
            return;

        GUIItem item = items.get(from);
        items.put(to, item);
        items.remove(from);
    }

    public boolean hasItem(int slot) {
        return items.containsKey(slot);
    }

    @Override
    public boolean execute(Player player, int slot, ClickType type) {
        if(player == null || slot >= getSize() || slot < 0 || type == null)
            return false;

        if(!items.containsKey(slot))
            return false;

        GUIItem item = items.get(slot);
        if(type == ClickType.RIGHT) {
            if(item.getActionRight() != null)
                item.getActionRight().click(player);

            return item.hasCancelRight();
        } else {
            if(item.getActionLeft() != null)
                item.getActionLeft().click(player);

            return item.hasCancelLeft();
        }
    }

    @Override
    public Inventory getInventory() {
        return inventory;
    }

}
