package me.paradoxpixel.themepark.listener;

import me.paradoxpixel.themepark.ThemeParkPlugin;
import me.paradoxpixel.themepark.api.attraction.Attraction;
import me.paradoxpixel.themepark.api.attraction.component.Status;
import me.paradoxpixel.themepark.api.event.attraction.AddAttractionEvent;
import me.paradoxpixel.themepark.api.event.attraction.RemoveAttractionEvent;
import me.paradoxpixel.themepark.api.event.attraction.StatusChangeEvent;
import me.paradoxpixel.themepark.api.event.region.AddRegionEvent;
import me.paradoxpixel.themepark.api.event.region.ChangeRegionEvent;
import me.paradoxpixel.themepark.api.event.region.RemoveRegionEvent;
import me.paradoxpixel.themepark.attraction.AttractionMenu;
import me.paradoxpixel.themepark.attraction.sign.SignManager;
import me.paradoxpixel.themepark.attraction.status.StatusManager;
import me.paradoxpixel.themepark.config.YamlConfig;
import me.paradoxpixel.themepark.utils.Message;
import me.paradoxpixel.themepark.utils.Utils;
import org.bukkit.Bukkit;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;

public class ChangeListener implements Listener {

    @EventHandler
    public void onChange(StatusChangeEvent e) {
        AttractionMenu.reload();
        SignManager.update(e.getAttraction());

        Attraction attraction = e.getAttraction();
        Status status = e.getStatusAfter();
        String message = Message.getMessage("attraction.changed.status." + status.toString());
        message = message.replace("{name}", attraction.getName());
        message = message.replace("{status}", StatusManager.getName(status));
        Bukkit.broadcastMessage(Utils.color(message));

        Bukkit.getScheduler().runTaskAsynchronously(ThemeParkPlugin.getInstance(), new Runnable() {
            @Override
            public void run() {
                ThemeParkPlugin.getInstance().getDatabaseHandler().updateAttraction(e.getAttraction());
            }
        });
    }

    @EventHandler
    public void onChange(AddAttractionEvent e) {
        AttractionMenu.reload();
        SignManager.loadSigns(e.getAttraction());

        Bukkit.getScheduler().runTaskAsynchronously(ThemeParkPlugin.getInstance(), new Runnable() {
            @Override
            public void run() {
                ThemeParkPlugin.getInstance().getDatabaseHandler().addAttraction(e.getAttraction());
            }
        });
    }

    @EventHandler
    public void onChange(RemoveAttractionEvent e) {
        AttractionMenu.reload();
        SignManager.remove(e.getAttraction());

        YamlConfig config = ThemeParkPlugin.getInstance().getAttraction();
        config.getConfig().set("attraction." + e.getAttraction().getId(), null);
        config.save();

        Bukkit.getScheduler().runTaskAsynchronously(ThemeParkPlugin.getInstance(), new Runnable() {
            @Override
            public void run() {
                ThemeParkPlugin.getInstance().getDatabaseHandler().removeAttraction(e.getAttraction());
            }
        });
    }

    @EventHandler
    public void onChange(AddRegionEvent e) {
        AttractionMenu.reload();

        Bukkit.getScheduler().runTaskAsynchronously(ThemeParkPlugin.getInstance(), new Runnable() {
            @Override
            public void run() {
                ThemeParkPlugin.getInstance().getDatabaseHandler().addRegion(e.getRegion());
            }
        });
    }

    @EventHandler
    public void onChange(ChangeRegionEvent e) {
        AttractionMenu.reload();

        Bukkit.getScheduler().runTaskAsynchronously(ThemeParkPlugin.getInstance(), new Runnable() {
            @Override
            public void run() {
                ThemeParkPlugin.getInstance().getDatabaseHandler().updateRegion(e.getRegion());
            }
        });
    }

    @EventHandler
    public void onChange(RemoveRegionEvent e) {
        AttractionMenu.reload();

        YamlConfig config = ThemeParkPlugin.getInstance().getAttraction();
        config.getConfig().set("region." + e.getRegion().getId(), null);
        config.save();

        Bukkit.getScheduler().runTaskAsynchronously(ThemeParkPlugin.getInstance(), new Runnable() {
            @Override
            public void run() {
                ThemeParkPlugin.getInstance().getDatabaseHandler().removeRegion(e.getRegion());
            }
        });
    }


}
