package me.paradoxpixel.themepark.listener;

import me.paradoxpixel.themepark.ThemeParkPlugin;
import me.paradoxpixel.themepark.api.API;
import me.paradoxpixel.themepark.api.attraction.Attraction;
import me.paradoxpixel.themepark.attraction.sign.SignManager;
import me.paradoxpixel.themepark.attraction.sign.StatusSign;
import me.paradoxpixel.themepark.attraction.status.StatusManager;
import me.paradoxpixel.themepark.config.YamlConfig;
import me.paradoxpixel.themepark.utils.Utils;
import org.bukkit.Location;
import org.bukkit.block.Block;
import org.bukkit.block.Sign;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.BlockBreakEvent;
import org.bukkit.event.block.SignChangeEvent;

public class SignListener implements Listener {

    private YamlConfig config = ThemeParkPlugin.getInstance().getSettings();
    private String name = config.getConfig().getString("sign.name");
    private String title = config.getConfig().getString("sign.title");

    @EventHandler
    public void onPlace(SignChangeEvent e) {
        if(!e.getLine(0).equalsIgnoreCase(name))
            return;

        String id = e.getLine(1);
        if(id == null || id.isEmpty() || !API.isAttraction(id))
            return;

        Attraction attraction = API.getAttraction(id);
        e.setLine(0, Utils.color(title));
        e.setLine(1, Utils.color(attraction.getName()));
        e.setLine(2, Utils.color(StatusManager.getName(attraction.getStatus())));

        Location location = e.getBlock().getLocation();
        StatusSign statusSign = new StatusSign(attraction, location);
        SignManager.addSign(statusSign);
    }

    @EventHandler
    public void onBreak(BlockBreakEvent e) {
        if(e.getBlock() == null)
            return;

        Block block = e.getBlock();
        if(!(block.getState() instanceof Sign))
            return;

        Sign sign = (Sign) block.getState();
        if(!sign.getLine(0).equals(Utils.color(title)))
            return;

        Attraction attraction = API.getAttractionFromName(sign.getLine(1));
        Location location = e.getBlock().getLocation();

        StatusSign s = new StatusSign(attraction, location);
        SignManager.removeSign(s);
    }

}
